

%macro portfolio_divs(xvar = ,
infile = umd,
retfile = rsch.msf,
datevar = rdate ,
k = 1 ,
sizevar = me ,
fffile = rsch.msi  ,
sizebreak = 80 ,
wind = 1 , p1 = 1 ,p2 = 99 ,
uscondsort = 0 , 
keeplc = 0 ,keepsc = 0 ,keepl  = 0 ,keeps  = 0 , keep_all = 0,
keepw = 1 ,
fill = 1 , 
keepsmb = 0); 


***************************************************************************
*                sort data      
************************************************************************** ; 
proc sort data = &infile out = temprr ;  by ex &datevar ; 
where ex is not missing and &datevar is not missing and &sizevar is not missing 
and &xvar is not missing ; 
run ; 

***************************************************************************
*               windosorize  
************************************************************************** ; 

%if  &wind = 1 %then %do ; 
%put "Windsorize at &p1 and &p2 level"; 
%windsor(dataset =  temprr, var= &xvar, byvar = ex &datevar , p1 = &p1 , p2 = &p2) ; 
proc sort data = temprr  ;  by ex &datevar ; run ;
%end ;

***************************************************************************
*  First 1: sort on size 
************************************************************************** ; 

proc sort data =  temprr out = temprr ;  by ex &datevar ; run ; 

* US size sort: NYSE-based ;  
proc univariate data=temprr  noprint;
  var &sizevar;
  where ex = 'USA' and exchcd  = 1 ; 
  by ex &datevar;
  output out=NYSE pctlpts = 50 pctlpre=P;run; 

* ex US size sort: top 20% by country ; 
proc univariate data=temprr  noprint;
  var &sizevar;
  where ex ^= 'USA' ; 
  by ex &datevar;
  output out=NYSE2 pctlpts = &sizebreak pctlpre=P;run; 

*merge breakpoints ; 
data NYSE ; 
set NYSE NYSE2(rename = (P&sizebreak = P50)) ; 
proc sort nodupkey ; 
by ex &datevar ; run ; 

*size ; 
data  temprr ; 
	merge  temprr nyse ; 
	by ex &datevar ; 
	if &sizevar >0 then do ; 
	nyse_size = 1 ; 
	if &sizevar >= P50 then nyse_size = 2 ;
	end ; 
	drop P50 ;
run ; 


***************************************************************************
*                   rank on second variable; 
************************************************************************** ;
proc sort data = temprr ; by ex &datevar  nyse_size ; run ; 

* US sort: nyse based (independent sort);   
%if &uscondsort = 0 %then %do ; 
proc univariate data= temprr noprint;
  var &xvar;
  where ex = 'USA' and exchcd  = 1 ; 
  by ex &datevar;
  output out=NYSE pctlpts = 30 70 pctlpre=P;run; 
  
*merge breakpoints ; 
data NYSEa ; set NYSE ; nyse_size = 1 ; run ; 
data NYSEb ; set NYSE ; nyse_size = 2 ; run ; 

data NYSE ;  set NYSEa NYSEb  ; proc sort nodupkey ; 
by ex &datevar nyse_size; 

proc delete data = NYSEa NYSEb ; run ; 

%end ; 

*US sort, conditional ; 
 %if &uscondsort = 1 %then %do ; 
proc univariate data= temprr noprint;
  var &xvar;
  where ex = 'USA' and exchcd  = 1 ; 
  by ex &datevar nyse_size;
  output out=NYSE pctlpts = 30 70 pctlpre=P;run; 
%end ; 

* International: Conditional sort ;  
proc univariate data= temprr noprint;
  var &xvar; 
  where  ex ^= 'USA' ; 
  by ex &datevar nyse_size;
  output out=NYSEint pctlpts = 30 70 pctlpre=P;run; 

*merge breakpoints ; 
data NYSE ; set NYSE NYSEint;  
proc sort nodupkey ; 
by ex &datevar nyse_size; run ; 

* Assign to ptf ; 
data temprr(keep = vid &datevar nyse_size ex BMR)  ; 
	merge temprr nyse ; 
	by ex &datevar nyse_size; 
	if &xvar ^=. and nyse_size >0 then do ; 
	BMR = 1 ; 
	if &xvar >  P30 then BMR = 2 ;
	if &xvar >= P70 then BMR = 3 ;
	drop P30 P70  ; 
	end ; 
run ; 



***************************************************************************
*              compute weights ;
************************************************************************** ;

proc sql ; 
create table dates as select distinct
&datevar as refresh_date format yymmddn8. label 'refresh_date', b.date as rdate format yymmddn8. label 'rdate' , c.date
from (select distinct &datevar from temprr) a , crsp.msi b , crsp.msi c
where  0 <= intck('month',a.&datevar, b.date) < 1 
and  0 <= intck('month',b.date, c.date) <= &k  order by refresh_date , date ; 

create view temprr2 as select distinct
a.vid , b.refresh_date , b.rdate , b.date , nyse_size ,  BMR , ex
from temprr a , dates b
where a.&datevar = b.refresh_date ; 

create  table w as select distinct
a.vid , a.refresh_date , a.rdate , a.date ,  a.ex , nyse_size ,  BMR , me / sum(me) as w format best12.
from temprr2 a , &retfile  b
where a.vid = b.vid and a.rdate = b.date and b.me > 0 
group by  a.ex , nyse_size ,  BMR , refresh_date  , a.rdate , a.date, b.date ; 


***************************************************************************
*              returns
*************************************************************************** ;

proc sql ; 
create table ptfret as select distinct
a.ex ,nyse_size ,  BMR , a.refresh_date , a.rdate , b.date, sum(w*ret) as ret, sum(w*retx) as retx, sum(w) as w
from w a , ind.dividends(keep = vid date ret retx)  b
where a.vid = b.vid and a.date = b.date
group by  a.ex , nyse_size ,  BMR , refresh_date  , a.rdate , b.date
order by  ex , nyse_size ,  BMR , refresh_date  , a.rdate , b.date ; 

drop view temprr2 ;
drop table dates ; 


***************************************************************************
*      Portfolios
*************************************************************************** ;

* ptfs ; 

data divs_&xvar; 
set ptfret;
run ; 


proc delete data = temprr _sl _sh ptfret _BH _BL _SM _BM nyse nyse2 nyseint  u; run ; 

%mend ; 

