clear all
set more off

* Load and clean returns
import excel "../../Excel results/p10.xlsx", firstrow case(l) clear
gen yrmo = ym(year(date),month(date))
format yrmo %tm
gen yrqtr = yq(year(date),quarter(date))
format yrqtr %tq
gen lret = ln(1+ret)
drop xret
reshape wide ret lret, i(yrmo) j(ind)
gen lmkt = ln(1+mkt+rf) // mkt is defined net of rf, so add back
tsset yrmo
collapse (sum) lret* lmkt*, by(yrqtr)
tsset yrqtr
gen mkt = exp(lmkt)-1
forvalues pnum=1/11{
	gen ret`pnum' = exp(lret`pnum')-1
}
save TMPData/p10_qtr, replace

* Load and clean consumption growth
import excel "../../Excel results/cg.xlsx", firstrow case(l) clear // cg is growth of PCE nondurable plus services, deflated by CPI (obtain from FRED)
gen yrqtr = yq(year(date),quarter(date))
format yrqtr %tq
tsset yrqtr
drop date
save TMPData/cg, replace

* Merge
use TMPData/p10_qtr, clear
drop in 1/2
merge 1:1 yrqtr using TMPData/cg
tsset yrqtr
forvalues pnum=1/11{
	gen ret`pnum'_un = ret`pnum'
	reg ret`pnum' mkt
	predict ret`pnum'_partial_mkt, residuals
}

* Cumulate
local window3 = 8 // quarters
foreach dgv of varlist cg lmkt lret* {
	qui tssmooth ma `dgv'_cum=`dgv'*(`window3'), window(0 0 `window3')
	replace `dgv'_cum = . if missing(F.`dgv') | missing(F`window3'.`dgv')
}
forvalues pnum=1/11 {
	gen ret`pnum'_cum = exp(lret`pnum'_cum)-1
}
gen mkt_cum = exp(lmkt_cum)-1

* Calculate and export covariances
foreach dgv of varlist cg mkt  {
	foreach adj in un partial_mkt {
		gen j=.
		label var j "Portfolio"
		gen cov_0 = .
		gen cov_24 = .
		label var cov_0 "Contemp. Cov w/ `dgv' (`adj')"
		label var cov_24 "2Y Cov w/ `dgv' (`adj')"
		gen cov_0_se = .
		label var cov_0_se "(Contemp. Cov SE)"
		gen cov_24_se = .
		label var cov_24_se "(2Y Cov SE)"

		qui sum `dgv'_cum
		local Tgmm = `r(N)'
		gen N = `Tgmm'
		label var N "N"
		local Sgmm_qs = min(ceil(1.95*(`Tgmm')^(1/3)),13) // from optimal calculation in LLSW for QS kernel truncation lag
		forvalues h=1/11{
			replace j=`h' in `h'
			qui correlate ret`h'_`adj' `dgv' if ~missing(`dgv'_cum), covariance
			qui replace cov_0 = `r(cov_12)' in `h'
			local covn1 = `r(N)'
			qui correlate ret`h'_`adj' `dgv'_cum if ~missing(`dgv'_cum), covariance
			qui replace cov_24 = `r(cov_12)' in `h'
			local covn2 = `r(N)'
			if regexm("`adj'","partial_mkt")==1{
				local adjv = "_partial_mkt"
			}
			if regexm("`adj'","un")==1{
				local adjv = ""			
			}
			gmm (ret`h'`adjv'-{mur}) (`dgv'-{mud}) ((`covn1'/(`covn1'-1))*(ret`h'`adjv'-{mur})*(`dgv'-{mud})-{cv_`dgv'_`h'_qtr}) if ~missing(`dgv'_cum), winitial(identity) vce(hac qu `Sgmm_qs') onestep // calculate standard errors using GMM covariance estimator with QS kernel
			qui replace cov_0_se = r(table)[2,3] in `h'
			gmm (ret`h'`adjv'-{mur}) (`dgv'_cum-{mud}) ((`covn2'/(`covn2'-1))*(ret`h'`adjv'-{mur})*(`dgv'_cum-{mud})-{cv_`dgv'_`h'_qtr}) if ~missing(`dgv'_cum), winitial(identity) vce(hac qu `Sgmm_qs') onestep		
			qui replace cov_24_se = r(table)[2,3] in `h'

		}
		tostring j, replace
		replace j = "10 - 1" in 11
		export excel j cov_0 cov_0_se cov_24 cov_24_se N using "../../Excel results/Covariances.xlsx" in 1/11, firstrow(varlabels) sheet("`dgv'_`adj'", modify) keepcellfmt 
		drop j cov_0 cov_24 cov_0_se cov_24_se N
	}
}

