clear all
set more off

* Load and clean
use "../Data/Ltg/realized_for_ses.dta", clear
split ind, gen(port) parse(__) destring
drop if port2 == 2 // keep only long and short legs
bysort refresh_date port2 year: egen div_avg = mean(div)
gen long_leg = i3.port2 // long_leg=1 for long leg (port2==3), 0 for short
gen size = i2.port1
local disc = 1.1
gen ddiv = div/(`disc'^year)
sort refresh_date long_leg size year
by refresh_date long_leg: egen ddiv_frac = pc(ddiv), prop

* Set up for SEs
egen panelid = group(year long_leg size)
gen ref_yr = year(refresh_date)
gen yr = ref_yr+year
format yr %ty
xtset panelid yr
local T=90
local llnw = round(`T'^(1/3))
local lag2 = 15

* Run and output
xtscc ddiv ibn.long_leg#ibn.year#ibn.size, noconstant lag(`lag2') ase
local exprs = ""
forvalues lon=0/1{
	local llong = 1-`lon'
	local denom = "(0"
	
	forvalues yrn=1/15 {
		local denom = "`denom'+(_b[`lon'.long_leg#`yrn'.year#0.size]+_b[`lon'.long_leg#`yrn'.year#1.size])/2"
	}
	local denom = "`denom')"
	
	forvalues yrn=1/15 {
		local exprs = "`exprs' (year`yrn'_long`llong': ((_b[`lon'.long_leg#`yrn'.year#0.size]+_b[`lon'.long_leg#`yrn'.year#1.size])/2)/`denom')"
	}
}
nlcom `exprs', post
outreg2 using "../Tables/Fig1aSEs.tex",  replace side noparen noaster  excel



